#
# Copyright (C) 2021 Jonathan Certes
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
#

set pagination off
set architecture armv7

# send commands to the remote monitor: openocd
monitor reset halt
monitor pld load 0 ./hw/design_0_wrapper.bit
monitor gdb_sync

# now controlling gdb:
file  sw/fsbl/main.elf
load  sw/fsbl/main.elf
break _boot
jump  _boot
layout regs

# add a breakpoint in each entry of the exception vectors table:
break *0x00
break *0x04
break *0x08
break *0x0c
break *0x10
break *0x14
break *0x18
break *0x1c

# run until we reach Loop (ps7_init terminated properly) or FsblHookFallback (an
# error occurred, case of qemu since all devices cannot be initialized):
break Loop
break FsblHookFallback
continue
continue

# load the code into the AXI slave:
monitor load_image sw/app/sw_axi.bin 0x40000000

# load the application:
file sw/app/main.elf
load sw/app/main.elf
break _start
jump  _start

# verify that there is nothing at the beginning of the exclusive stack:
printf "witness @"
x /1wx 0x42000000

# run startup.s:
break main
continue

# execute the code from the AXI slave:
break __my_breakpoint
continue

# we have pushed 13 times into the exclusive stack:
# - lr and (fp, sp) before switching stack context
# - values from 0 to 9 in a loop
printf "end of the exclusive stack:\n"
x /13wx 0x42000fcc

# show the witness (beginning of the exclusive stack):
printf "witness @"
x /1wx 0x42000000

# This is it for the execution. Now we show that branch destinations must be
# aligned to 0x20 in an AXI slave:

continue
# next breakpoint is at 0x0c (entry point for prefetch abort)
printf "Address causing the prefetch abort: 0x%x\n", ($lr - 4)

# That's all folks!
