
#
# Copyright (C) 2021 Jonathan Certes
#
# This program is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software
# Foundation, either version 3 of the License, or (at your option) any later
# version.
#
# This program is distributed in the hope that it will be useful, but WITHOUT
# ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along with
# this program. If not, see <http://www.gnu.org/licenses/>.
#

#
# This script must be sourced when a bd_design has been created.
#

set_property source_mgmt_mode All [current_project]

## IMPORTANT! Use digilent pre-configuration for the board, see
## vivadoMakefile.tcl, option --board.

##
# adding zynq processing system to the design:
#
set theInstName "processing_system7_0"
create_bd_cell -type "ip" \
               -vlnv "xilinx.com:ip:processing_system7:5.5" \
               ${theInstName}
# run block automation:
apply_bd_automation -rule "xilinx.com:bd_rule:processing_system7" -config {
  make_external      "FIXED_IO, DDR"
  apply_board_preset 1
  Master             "Disable"
  Slave              "Disable"
} [get_bd_cells ${theInstName}]
# connect clock:
connect_bd_net [get_bd_pins "${theInstName}/FCLK_CLK0"] \
               [get_bd_pins "${theInstName}/M_AXI_GP0_ACLK"]
# save its instance name:
set theZynq ${theInstName}


foreach theSlaveName { software stack } {

  ##
  # instantiate a BRAM to store the slave
  set theInstName "i_${theSlaveName}"
  create_bd_cell -type "module" -reference "slave" ${theInstName}
  # saving its instance name:
  set theSlave ${theInstName}

  ##
  # adding a BRAM controller:
  set theInstName "i_bram_ctrl_${theSlaveName}"
  create_bd_cell -type "ip" \
                 -vlnv "xilinx.com:ip:axi_bram_ctrl:4.1" \
                 ${theInstName}
  # setting it to single port AXI Lite:
  set_property -dict {
    CONFIG.SINGLE_PORT_BRAM 1
    CONFIG.PROTOCOL         AXI4LITE
    CONFIG.ECC_TYPE         0
  } [get_bd_cells ${theInstName}]
  # connect clock:
  connect_bd_net [get_bd_pins "${theZynq}/FCLK_CLK0"] \
                 [get_bd_pins "${theInstName}/s_axi_aclk"]
  # saving its instance name:
  set theBramController ${theInstName}

  ##
  # run block automation:
  apply_bd_automation -rule xilinx.com:bd_rule:axi4 -config [list \
    Clk_master "/${theZynq}/FCLK_CLK0 (100 MHz)"                  \
    Clk_slave  "/${theZynq}/FCLK_CLK0 (100 MHz)"                  \
    Clk_xbar   "/${theZynq}/FCLK_CLK0 (100 MHz)"                  \
    Master     "/${theZynq}/M_AXI_GP0"                            \
    Slave      "/${theBramController}/S_AXI"                      \
    ddr_seg    "Auto"                                             \
    intc_ip    "New AXI SmartConnect"                             \
    master_apm 0
  ] [get_bd_intf_pins "${theBramController}/S_AXI"]


  # changes the addresses range according to the infered BRAM:
  set theRange "4K"
  set_property range ${theRange} \
    [get_bd_addr_segs "${theZynq}/Data/SEG_${theBramController}_Mem0"]

  ##
  # connections between the BRAM controller and the slave:
  set thePort "a" ;# either "a" or "b"
  connect_bd_net [get_bd_pins "${theBramController}/bram_addr_${thePort}"]   \
                 [get_bd_pins "${theSlave}/addr"]
  connect_bd_net [get_bd_pins "${theBramController}/bram_clk_${thePort}"]    \
                 [get_bd_pins "${theSlave}/clk"]
  connect_bd_net [get_bd_pins "${theBramController}/bram_rddata_${thePort}"] \
                 [get_bd_pins "${theSlave}/rddata"]
  connect_bd_net [get_bd_pins "${theBramController}/bram_wrdata_${thePort}"] \
                 [get_bd_pins "${theSlave}/wrdata"]
  connect_bd_net [get_bd_pins "${theBramController}/bram_we_${thePort}"] \
                 [get_bd_pins "${theSlave}/we"]
  connect_bd_net [get_bd_pins "${theBramController}/bram_rst_${thePort}"] \
                 [get_bd_pins "${theSlave}/rst"]

  regenerate_bd_layout
}

