/*
 * Copyright (C) 2021 Jonathan Certes
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <http://www.gnu.org/licenses/>.
 */

.text
  /**
   * /!\ Caution! Branch destinations must be aligned to 32 Bytes (8
   * instructions in ARM state) otherwise we get a prefetch abort exception.
   */

  // saving stack context + switching to exclusive stack:
  mov r0, fp
  mov r1, sp
  movw r2, #0x1000  // __ex_stack_top
  movt r2, #0x4200  //
  mov fp, r2
  mov sp, r2
  push {lr}
  push {r0}

  push {r1}
  nop
  nop
  nop
  nop
  nop
  nop
  mov r0, #-1   // set r0 to -1

  // push from 0 to 9 to the stack:
  my_loop_0:
    add r0, r0, #1
    push {r0}
    cmp r0, #9
    blt my_loop_0
  nop
  nop
  nop
  mov r0, #-1

  // pop 10 times from the stack:
  my_loop_1:
    add r0, r0, #1
    pop {r1}
    cmp r0, #9
    blt my_loop_1
  nop
  nop
  nop
  nop

  // witness (0x42000000 == __ex_stack_low):
  movw r3, #0x0000
  movt r3, #0x4200
  movw r2, #0x5678
  movt r2, #0x1234
  str r2, [r3]
  // restoring stack context and old link:
  pop {r1}
  pop {r0}
  pop {lr}

  mov fp, r0
  mov sp, r1
  bx lr // return
  nop
  nop
  nop
  nop
  nop

  // here we illustrate that a branch to a non-aligned address causes a prefetch
  // abort exception (entry in the vector table is at 0x0c):
  abort_example:
    // indirect branch:
    add r0, pc, #0
    mov pc, r0
    // <- destination for the indirect branch (abort_example + 8)

